/**
  *  \file game/load-host.cc
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "game/load.h"
#include "util/console.h"
#include "util/nls.h"
#include "util/except.h"
#include "game/global.h"
#include "io/dirs.h"

static void
loadHostMinefields(GGameTurn& trn, Stream& s, int first_id, int count, int /*who*/)
{
    /* We're tolerant with this file, as it's not mandatory. */
    for (int i = 0; i < count; ++i) {
        THostMinefield mf;
        if (!getStructure(s, mf))
            break;
        if (mf.owner != 0) {
            TMinefieldReport report;
            report.x               = mf.x;
            report.y               = mf.y;
            report.owner           = mf.owner;
            report.web             = mf.type;
            report.units_known     = true;
            report.type_known      = true;
            report.radius_or_units = mf.units;
            report.turn            = trn.getTurnNumber();
            report.why             = GMinefield::mfa_Scanned;
            trn.getCurrentUniverse().ty_minefields.addMinefieldReport(first_id, report);
        }
        ++first_id;
    }
}

void
loadHostDirectory(GGameTurn& trn, Directory& d)
{
    TInt16 n;

    console.write(LOG_INFO, _("Loading host data..."));

    /* FIXME: somewhere export the fact that NUM_OWNERS means we're host */
    GPlayerSet players = GPlayerSet::allUpTo(NUM_OWNERS);

    /* Mark everyone as active */
    for (int i = 1; i <= NUM_PLAYERS; ++i)
        trn.setHaveData(i);

    /* Ships */
    Ptr<Stream> s = d.openFile("ship.hst", Stream::C_READ);
    int num_ships = (s->getSize() - 2) / TShip::size;
    if (num_ships < 500)
        throw FileTooShortException(*s);
    if (num_ships > NUM_SHIPS)
        num_ships = NUM_SHIPS;
    getStructureT(*s, n);
    loadShips(trn, *s, num_ships, LoadBoth, players);

    /* Planets */
    s = d.openFile("pdata.hst", Stream::C_READ);
    getStructureT(*s, n);
    loadPlanets(trn, *s, NUM_PLANETS, LoadBoth, players);

    /* Bases */
    s = d.openFile("bdata.hst", Stream::C_READ);
    getStructureT(*s, n);
    loadBases(trn, *s, NUM_PLANETS, LoadBoth, players);

    /* Minefields */
    s = d.openFileNT("mines.hst", Stream::C_READ);
    if (s)
        loadHostMinefields(trn, *s, 1, 500, LoadBoth);
    s = d.openFileNT("extmines.hst", Stream::C_READ);
    if (s)
        loadHostMinefields(trn, *s, 501, 10000-500, LoadBoth);
}
