/**
  *  \file int/if/specif.h
  *  \brief Interpreter Game Interface: Specification Access
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_IF_SPECIF_H
#define PCC_V2_INT_IF_SPECIF_H

#include "cpluslib/types.h"
#include "int/context.h"

class GComponent;
class GEngine;
class GWeapon;
class IntArgBlock;
class IntValue;

IntValue*   IFTorpedoGet(IntArgBlock& args);
IntContext* IFTorpedoMake();
int32_t     IFTorpedoDim(int32_t dim);
IntValue*   IFLauncherGet(IntArgBlock& args);
IntContext* IFLauncherMake();
IntValue*   IFBeamGet(IntArgBlock& args);
IntContext* IFBeamMake();
int32_t     IFBeamDim(int32_t dim);
IntValue*   IFEngineGet(IntArgBlock& args);
IntContext* IFEngineMake();
int32_t     IFEngineDim(int32_t dim);

class IntTorpedoContext : public IntContext {
    int id;
 public:
    IntTorpedoContext(int id);
    ~IntTorpedoContext();
    int32_t     lookup(const IntNameQuery& name);
    void        set(int32_t index, IntValue* value);
    IntValue*   get(int32_t index);
    bool        next();
    IntContext* clone();
    GObject*    getObject();
    string_t    toString(bool readable);
    void        store(IntTagNode& sv, Stream& aux);
    void        enumProperties(IntPropertyAcceptor& acceptor);
};

class IntLauncherContext : public IntContext {
    int id;
 public:
    IntLauncherContext(int id);
    ~IntLauncherContext();
    int32_t     lookup(const IntNameQuery& name);
    void        set(int32_t index, IntValue* value);
    IntValue*   get(int32_t index);
    bool        next();
    IntContext* clone();
    GObject*    getObject();
    string_t    toString(bool readable);
    void        store(IntTagNode& sv, Stream& aux);
    void        enumProperties(IntPropertyAcceptor& acceptor);
};

class IntBeamContext : public IntContext {
    int id;
 public:
    IntBeamContext(int id);
    ~IntBeamContext();
    int32_t     lookup(const IntNameQuery& name);
    void        set(int32_t index, IntValue* value);
    IntValue*   get(int32_t index);
    bool        next();
    IntContext* clone();
    GObject*    getObject();
    string_t    toString(bool readable);
    void        store(IntTagNode& sv, Stream& aux);
    void        enumProperties(IntPropertyAcceptor& acceptor);
};

class IntEngineContext : public IntContext {
    int id;
 public:
    IntEngineContext(int id);
    ~IntEngineContext();
    int32_t     lookup(const IntNameQuery& name);
    void        set(int32_t index, IntValue* value);
    IntValue*   get(int32_t index);
    bool        next();
    IntContext* clone();
    GObject*    getObject();
    string_t    toString(bool readable);
    void        store(IntTagNode& sv, Stream& aux);
    void        enumProperties(IntPropertyAcceptor& acceptor);
};

/** Property for a GComponent. */
enum IntComponentProperty {
    icpMass,
    icpTech,
    icpCostT,
    icpCostD,
    icpCostM,
    icpCostMC,
    icpCostSup,
    icpCostStr,
    icpName,
    icpNameShort,
    icpId
};

/** Property for a GWeapon. */
enum IntWeaponProperty {
    iwpKill,
    iwpDamage
    // TorpCost? PCC 1.x doesn't have it.
};

/** Property for a GEngine. */
enum IntEngineProperty {
    iepEfficientWarp,
    iepFuelFactor
};

IntValue* getComponentProperty(const GComponent& comp, IntComponentProperty icp);
void      setComponentProperty(GComponent& comp, IntComponentProperty icp, IntValue* value);
IntValue* getWeaponProperty(const GWeapon& w, IntWeaponProperty iwp, bool isTorpedo);
IntValue* getEngineProperty(const GEngine& e, IntEngineProperty iep);
void      setEngineProperty(GEngine& e, IntEngineProperty iep, IntValue* value);

#endif
