/**
  *  \file int/special.h
  *  \brief Interpreter: Special Commands
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_SPECIAL_H
#define PCC_V2_INT_SPECIAL_H

#include "cpluslib/string.h"

class IntTokenizer;
class IntBytecodeObject;
class IntStatementCompilationContext;
class IntPropertyAcceptor;

/** Interface for a special commands. Special commands are compiled
    specially. They are looked up in a special symbol table. Objects
    of this type are made known to the compiler using the
    defineSpecialCommand function. For the user, special commands
    behave identical to commands implemented within the compiler core
    (statement.cc). */
class IntSpecialCommand {
 public:
    virtual ~IntSpecialCommand();
    virtual void compileCommand(IntTokenizer& line, IntBytecodeObject& bco, const IntStatementCompilationContext& scc) = 0;
};

/** Simple special command. This hands compilation to a static
    function. */
class IntSimpleSpecialCommand : public IntSpecialCommand {
 public:
    typedef void (*compile_t)(IntTokenizer& line, IntBytecodeObject& bco, const IntStatementCompilationContext& scc);

    IntSimpleSpecialCommand(compile_t function);
    ~IntSimpleSpecialCommand();

    void compileCommand(IntTokenizer& line, IntBytecodeObject& bco, const IntStatementCompilationContext& scc);

 private:
    compile_t function;
};



void defineSpecialCommand(const char* name, IntSpecialCommand* newCmd);

IntSpecialCommand* lookupSpecialCommand(string_t name);
void enumSpecialCommands(IntPropertyAcceptor& acceptor);

#endif
