/**
  *  \file reshack/tools.h
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_RESHACK_TOOLS_H
#define PCC_V2_RESHACK_TOOLS_H

#include "cpluslib/object.h"
#include "cpluslib/string.h"
#include "gfx/gfxutil.h"
#include "gfx/pixmap.h"
#include "util/nls.h"

class GfxContext;
class UIWidget;

class RHTool : public Object {
    const bool needs_preview;
    const string_t name;
 public:
    RHTool(bool needs_preview, string_t name);
    ~RHTool();

    virtual void click(GfxContext& ctx, GfxPoint pt, uint32_t bg) = 0;
    virtual void drag(GfxContext& ctx, GfxPoint pt) = 0;
    virtual void release(GfxContext& ctx, GfxPoint pt) = 0;
    virtual bool isUsable();

    bool needsPreview() const;
    const string_t& getName() const;
};


class RHPencilTool : public RHTool {
    GfxPoint prev;
 public:
    RHPencilTool()
        : RHTool(false, _("Pencil"))
        { }
    void click(GfxContext& c, GfxPoint pt, uint32_t bg);
    void drag(GfxContext& c, GfxPoint pt);
    void release(GfxContext& c, GfxPoint pt);
};

class RHBrushTool : public RHTool {
 public:
    RHBrushTool()
        : RHTool(false, _("Brush"))
        { }
    void click(GfxContext& c, GfxPoint pt, uint32_t bg);
    void drag(GfxContext& c, GfxPoint pt);
    void release(GfxContext& c, GfxPoint pt);
};

class RHLineTool : public RHTool {
    GfxPoint prev;
 public:
    RHLineTool()
        : RHTool(true, _("Line"))
        { }
    void click(GfxContext& c, GfxPoint pt, uint32_t bg);
    void drag(GfxContext& c, GfxPoint pt);
    void release(GfxContext& c, GfxPoint pt);
};

class RHRectangleTool : public RHTool {
    GfxPoint prev;
 public:
    RHRectangleTool()
        : RHTool(true, _("Rectangle"))
        { }
    void click(GfxContext& c, GfxPoint pt, uint32_t bg);
    void drag(GfxContext& c, GfxPoint pt);
    void release(GfxContext& c, GfxPoint pt);
};

class RHBarTool : public RHTool {
    GfxPoint prev;
 public:
    RHBarTool()
        : RHTool(true, _("Solid Bar"))
        { }
    void click(GfxContext& c, GfxPoint pt, uint32_t bg);
    void drag(GfxContext& c, GfxPoint pt);
    void release(GfxContext& c, GfxPoint pt);
};

class RHCircleTool : public RHTool {
    GfxPoint prev;
 public:
    RHCircleTool()
        : RHTool(true, _("Circle"))
        { }
    void click(GfxContext& c, GfxPoint pt, uint32_t bg);
    void drag(GfxContext& c, GfxPoint pt);
    void release(GfxContext& c, GfxPoint pt);
};

class RHCopyTool : public RHTool {
    GfxPoint prev;
    UIWidget* cut_notify;
    uint32_t bg_color;
 public:
    RHCopyTool(UIWidget* cut_notify)
        : RHTool(true, _("Copy")), cut_notify(cut_notify), bg_color(0)
        { }
    void click(GfxContext& c, GfxPoint pt, uint32_t bg);
    void drag(GfxContext& c, GfxPoint pt);
    void release(GfxContext& c, GfxPoint pt);
};

class RHPasteTool : public RHTool {
 public:
    RHPasteTool(bool single)
        : RHTool(single, single ? _("Paste") : _("Stamp"))
        { }
    void click(GfxContext& c, GfxPoint pt, uint32_t bg);
    void drag(GfxContext& c, GfxPoint pt);
    void release(GfxContext& c, GfxPoint pt);
    bool isUsable();
};

class RHPipetteTool : public RHTool {
 public:
    RHPipetteTool()
        : RHTool(false, _("Pipette"))
        { }
    void click(GfxContext& c, GfxPoint pt, uint32_t bg);
    void drag(GfxContext& c, GfxPoint pt);
    void release(GfxContext& c, GfxPoint pt);
};

Ptr<GfxPixmap> getClipboardImage();



inline bool
RHTool::needsPreview() const
{
    return needs_preview;
}

inline const string_t&
RHTool::getName() const
{
    return name;
}

#endif
