/**
  *  \file ui/widget-busy.cc
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "ui/widget-busy.h"
#include "ui/window.h"
#include "util/nls.h"
#include "gfx/palette.h"
#include "ui/uiglobal.h"
#include "ui/util.h"
#include "ui/event.h"

/** Constructor. */
UIBusyWidget::UIBusyWidget()
    : UIFixedWidget(0),
      phase(0), last_time(0)
{
    int width = fonts[FONT_TITLE]->getTextWidth(_("Wait..."));
    int height = font_heights[FONT_TITLE];

    setExtent(GfxRect(0, 0, width + 16, height + 8));
    ui_root->centerWidget(*this);
}

/** Destructor. */
UIBusyWidget::~UIBusyWidget()
{ }

/** Draw the widget's frame. */
void
UIBusyWidget::drawFrame(GfxCanvas& can)
{
    GfxRect r = getExtent();
    r.grow(-3, -3);
    drawDashedFrame(can, r, phase, standard_colors[COLOR_YELLOW], standard_colors[COLOR_GRAYSCALE+3]);
}

void
UIBusyWidget::drawContent(GfxCanvas& can)
{
    GfxRect r = getExtent();
    drawFrameUp(can, r);
    r.grow(-1, -1);
    drawSolidBar(can, r, standard_colors[COLOR_GRAYSCALE+3]);

    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(GfxStandardColorScheme::instance);
    ctx.setColor(COLOR_WHITE);
    ctx.setTextAlign(1, 1);
    ctx.useFont(*fonts[FONT_TITLE]);
    outText(ctx, r.x + r.w/2, r.y + r.h/2, _("Wait..."));

    drawFrame(can);

    can.update(getExtent());
}

/** Advance time. This causes the yellow frame to advance to indicate
    activity. We limit movement to a rate of 20 Hz, to make it more
    visually appealing. */
void
UIBusyWidget::tick()
{
    long now = getTimer();
    if (now > last_time + TIMER_HZ/20) {
        last_time = now;
        --phase;
        if (UIBaseWidget* g = getParent()) {
            ObjectHolder h;
            GfxCanvas& can = g->getCanvasFor(*this, h);
            can.lock();
            drawFrame(can);
            can.unlock();
            can.update(getExtent());
        }
    }
}
